#! /usr/bin/env python3

import os
import subprocess
import sys
sys.path.insert(1, '../scripts')
from launchSlurmArray import launchSlurmArray

###############################################################################
def printUsageAndExit() :
  print("USAGE : %s (train | eval) (gpu | cpu) batchesDescription.py nbHours jobName maxNbSimultaneousJobs"%sys.argv[0], file=sys.stderr)
  exit(1)
###############################################################################

###############################################################################
def prepareExperiment(lang, template, expName) :
  subprocess.Popen("./prepareExperiment.sh %s %s %s"%(lang,template,expName),
      shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE).wait()
###############################################################################

###############################################################################
def addNamesAndCommandsTrain(names, commands, mode, expName, arguments, seed) :
  names.append(expName)
  
  commands.append("./train.sh {} bin/{} {} --silent --seed {}".format(mode, expName,arguments,seed))
###############################################################################

###############################################################################
def addNamesAndCommandsDecode(names, commands, mode, expName) :
  names.append(expName)
  
  commands.append("./evaluate.sh {} bin/{} --silent".format(mode, expName))
###############################################################################

###############################################################################
if __name__ == "__main__" :

  if len(sys.argv) != 7 :
    printUsageAndExit()

  mode = sys.argv[1]
  device = sys.argv[2]
  batchesDescription = sys.argv[3]
  nbHours = sys.argv[4]
  name = sys.argv[5]
  limit = sys.argv[6]

  if mode not in ["train","eval"] or device not in ["cpu","gpu"] :
    printUsageAndExit()

  desc = __import__(os.path.splitext(batchesDescription)[0])

  names = []
  commands = []

  for lang in desc.langs :
    for xp in desc.templatesExperiments :
      for i in desc.repRange :
        xp['lang'] = lang
        xp['expName'] = xp['expName'].split('.')[0]+"."+lang+"."+str(i)
        if mode == "train" :
          prepareExperiment(xp['lang'],xp['template'],xp['expName'])
          addNamesAndCommandsTrain(names, commands, xp['mode'],xp['expName'],xp['arguments'],seed=100+i)
        else :
          addNamesAndCommandsDecode(names, commands, xp['mode'],xp['expName'])

  launchSlurmArray(names, commands, name, device, nbHours, limit)
###############################################################################

